import { csv, select } from 'd3';
import { scatterPlot } from './scatterplot.js';

const csvUrl =
  'https://gist.githubusercontent.com/Mahesh-Polisetty/14316699380f4bc1beb720eb5ccd7fa9/raw/13f0454025771f8ed9d8deb145a5c6541b893fce/gistfile1.txt';

// Parsing function to convert string values to numbers
const parseRow = (d) => {
  return {
    State: d.State,
    2020: +d['2020 Revenue (USD)'],
    2021: +d['2021 Revenue (USD)'],
    2022: +d['2022 Revenue (USD)'],
    2023: +d['2023 Revenue (USD)'],
  };
};

const width = window.innerWidth * 0.8;
const height = window.innerHeight * 0.8;
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Function to define the main logic
const main = async () => {
  // Creating the scatter plot with initial configuration
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d['2023']) // X-axis: 2023 revenue
    .yValue((d) => d['2020']) // Y-axis: 2020 revenue
    .margin({
      top: 40,
      right: 40,
      bottom: 60,
      left: 60,
    })
    .radius(6); // Adjusting the radius of data points

  svg.call(plot);

  const columns = ['2020', '2021', '2022', '2023']; // List of years for animation
  let i = 0;
  // Setting a lower interval for smoother animation
  setInterval(() => {
    plot
      .xValue((d) => d[columns[i % columns.length]]) // Update x-value for animation
      .symbol((d, i) => d3.symbols[i % d3.symbols.length]); // Cycle through different symbols
    svg.call(plot);
    i++;
  }, 1000); // Lower interval between updates
};

main(); // Execute the main function
