import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  symbol,
  symbolCircle,
  symbolCross,
  symbolDiamond,
  symbolSquare,
  symbolStar,
  symbolTriangle,
  symbolWye,
} from 'd3'; // Importing necessary D3 functions

export const scatterPlot = () => {
  // Define variables to be used in the scatter plot
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;
  let symbolType = symbolCircle; // Default symbol type

  // Main function to generate scatter plot
  const my = (selection) => {
    const x = scaleLinear() // X-axis scale
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear() // Y-axis scale
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
    }));

    // Data points with different symbols
    const circles = selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('transform', (d) => `translate(${d.x}, ${d.y})`) // Translate to position
      .attr('d', symbol().type(symbolType).size(radius)); // Define symbol type and size

    // X-axis
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0, ${height - margin.bottom})`,
      )
      .call(axisBottom(x));

    // Y-axis
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left}, 0)`)
      .call(axisLeft(y));
  };

  // Getter and setter methods for configuration options
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  my.symbol = function (_) {
    return arguments.length
      ? ((symbolType = _), my)
      : symbolType;
  };

  return my;
};
